@@ -210,6 +210,10 @@ class RayHybridRouter:
         self.rpc_backends = rpc_backends or []
         self.has_rpc_backends = len(self.rpc_backends) > 0
 
+        # Log SOLLOL version at initialization
+        from sollol import __version__
+        logger.info(f"📦 SOLLOL v{__version__} - RayHybridRouter initializing")
+
         # Initialize Ray with dashboard enabled (for Ollama pool parallelization even without RPC)
         if self.enable_distributed:
             if not ray.is_initialized():
@@ -219,6 +223,10 @@ class RayHybridRouter:
                     dashboard_host="0.0.0.0",
                     dashboard_port=8265,
                     include_dashboard=True,
+                    # Conservative memory settings to prevent OOM kills
+                    object_store_memory=500 * 1024 * 1024,  # 500MB object store
+                    _memory=2 * 1024 * 1024 * 1024,  # 2GB total per worker
+                    num_cpus=2,  # Limit parallelism to reduce memory pressure
                 )
                 logger.info("📊 Ray dashboard available at http://localhost:8265")
 
