@@ -120,7 +120,20 @@ class UnifiedDashboard:
             try:
                 from dask.distributed import Client
                 self.dask_client = Client()
-                logger.info(f"📊 Dask client initialized - dashboard at http://localhost:{dask_dashboard_port}")
+
+                # Get actual dashboard URL from client (may be on different port if 8787 was taken)
+                if hasattr(self.dask_client, 'dashboard_link'):
+                    actual_dashboard_url = self.dask_client.dashboard_link
+                    # Extract port from URL like http://127.0.0.1:45423/status
+                    import re
+                    port_match = re.search(r':(\d+)', actual_dashboard_url)
+                    if port_match:
+                        self.dask_dashboard_port = int(port_match.group(1))
+                        logger.info(f"📊 Dask client initialized - dashboard at {actual_dashboard_url}")
+                    else:
+                        logger.info(f"📊 Dask client initialized - dashboard at http://localhost:{dask_dashboard_port}")
+                else:
+                    logger.info(f"📊 Dask client initialized - dashboard at http://localhost:{dask_dashboard_port}")
             except Exception as e:
                 logger.warning(f"⚠️  Could not initialize Dask client: {e}")
 
@@ -158,8 +171,9 @@ class UnifiedDashboard:
         def metrics():
             """Get SOLLOL metrics."""
             try:
-                if self.router:
-                    stats = asyncio.run(self.router.get_stats())
+                if self.router and hasattr(self.router, 'get_stats'):
+                    # get_stats() is not async - call it directly
+                    stats = self.router.get_stats()
                 else:
                     stats = {}
 
@@ -176,20 +190,32 @@ class UnifiedDashboard:
             """Get all Ollama nodes in the network (universal)."""
             try:
                 # Try to get from router's pool
-                if self.router and hasattr(self.router, 'ollama_pool'):
+                if self.router and hasattr(self.router, 'ollama_pool') and self.router.ollama_pool:
                     pool = self.router.ollama_pool
                     nodes = []
                     for node in pool.nodes:
-                        nodes.append({
-                            "url": node.url,
-                            "healthy": node.healthy,
-                            "last_latency_ms": node.last_latency_ms,
-                            "failure_count": node.failure_count,
-                            "models": node.models,
-                            "total_vram_mb": node.total_vram_mb,
-                            "free_vram_mb": node.free_vram_mb,
-                            "status": "healthy" if node.healthy else "unhealthy",
-                        })
+                        # Handle both OllamaNode objects and dict formats
+                        if isinstance(node, dict):
+                            url = f"http://{node.get('host', 'localhost')}:{node.get('port', 11434)}"
+                            nodes.append({
+                                "url": url,
+                                "status": "healthy",
+                                "latency_ms": 0,
+                                "load_percent": 0,
+                                "memory_mb": 0,
+                            })
+                        else:
+                            # OllamaNode object
+                            nodes.append({
+                                "url": node.url,
+                                "healthy": node.healthy,
+                                "last_latency_ms": node.last_latency_ms,
+                                "failure_count": node.failure_count,
+                                "models": node.models,
+                                "total_vram_mb": node.total_vram_mb,
+                                "free_vram_mb": node.free_vram_mb,
+                                "status": "healthy" if node.healthy else "unhealthy",
+                            })
                     return jsonify({"nodes": nodes, "total": len(nodes)})
                 else:
                     # Fallback: auto-discover nodes and transform to expected format
@@ -311,6 +337,16 @@ class UnifiedDashboard:
             """Get distributed traces."""
             return jsonify(list(self.trace_history))
 
+        @self.app.route("/api/dashboard/config")
+        def dashboard_config():
+            """Get dashboard configuration (actual ports, etc)."""
+            return jsonify({
+                "ray_dashboard_port": self.ray_dashboard_port,
+                "dask_dashboard_port": self.dask_dashboard_port,
+                "ray_dashboard_url": f"http://localhost:{self.ray_dashboard_port}",
+                "dask_dashboard_url": f"http://localhost:{self.dask_dashboard_port}",
+            })
+
         @self.app.route("/api/ray/metrics")
         def ray_metrics():
             """Get Ray metrics."""
@@ -947,11 +983,12 @@ UNIFIED_DASHBOARD_HTML = """
         .container {
             display: grid;
             grid-template-columns: 1fr 1fr 1fr;
-            grid-template-rows: auto auto auto 1fr;
+            grid-template-rows: auto auto auto auto auto;
             gap: 1rem;
             padding: 1rem;
-            height: calc(100vh - 120px);
+            max-height: calc(100vh - 120px);
             overflow-y: auto;
+            overflow-x: hidden;
         }
         .metrics-bar {
             grid-column: 1 / -1;
@@ -984,6 +1021,13 @@ UNIFIED_DASHBOARD_HTML = """
             overflow: hidden;
             display: flex;
             flex-direction: column;
+            min-height: 250px;
+            max-height: 400px;
+        }
+        .dashboard-panel.full-height {
+            min-height: 500px;
+            max-height: 500px;
+            height: 500px;
         }
         .panel-header {
             background: #334155;
@@ -993,7 +1037,12 @@ UNIFIED_DASHBOARD_HTML = """
         }
         .panel-content {
             flex: 1;
-            overflow: hidden;
+            overflow: auto;
+            min-height: 200px;
+        }
+        .full-height .panel-content {
+            height: 450px;
+            min-height: 450px;
         }
         iframe {
             width: 100%;
@@ -1015,6 +1064,7 @@ UNIFIED_DASHBOARD_HTML = """
             padding: 0.75rem;
             margin: 0.5rem;
             border-left: 3px solid #10b981;
+            min-height: 60px;
         }
         .node-card.offline, .backend-card.offline {
             border-left-color: #ef4444;
@@ -1025,10 +1075,12 @@ UNIFIED_DASHBOARD_HTML = """
             color: #a78bfa;
             font-size: 0.9rem;
             margin-bottom: 0.5rem;
+            word-break: break-all;
         }
         .node-stats, .backend-stats {
             display: flex;
-            gap: 1rem;
+            gap: 0.5rem;
+            flex-wrap: wrap;
             font-size: 0.75rem;
             color: #94a3b8;
         }
@@ -1036,6 +1088,7 @@ UNIFIED_DASHBOARD_HTML = """
             background: #1e293b;
             padding: 0.25rem 0.5rem;
             border-radius: 0.25rem;
+            white-space: nowrap;
         }
         .status-badge {
             display: inline-block;
@@ -1043,9 +1096,29 @@ UNIFIED_DASHBOARD_HTML = """
             border-radius: 0.25rem;
             font-size: 0.7rem;
             font-weight: 600;
+            white-space: nowrap;
         }
         .status-healthy { background: #10b981; color: #fff; }
         .status-offline { background: #ef4444; color: #fff; }
+        .compact-table {
+            width: 100%;
+            border-collapse: collapse;
+            font-size: 0.8rem;
+        }
+        .compact-table th {
+            background: #334155;
+            padding: 0.5rem;
+            text-align: left;
+            font-weight: 600;
+            border-bottom: 2px solid #667eea;
+        }
+        .compact-table td {
+            padding: 0.5rem;
+            border-bottom: 1px solid #334155;
+        }
+        .compact-table tr:hover {
+            background: #334155;
+        }
     </style>
 </head>
 <body>
@@ -1147,29 +1220,50 @@ UNIFIED_DASHBOARD_HTML = """
         </div>
 
         <!-- Ray Dashboard -->
-        <div class="dashboard-panel">
+        <div class="dashboard-panel full-height">
             <div class="panel-header">
                 <span class="status-indicator" id="ray-status"></span>
                 Ray Dashboard
             </div>
-            <div class="panel-content">
-                <iframe src="http://localhost:8265" id="ray-iframe"></iframe>
+            <div class="panel-content" id="ray-content">
+                <div style="color: #94a3b8; text-align: center; padding: 2rem;">
+                    Checking Ray availability...
+                </div>
             </div>
         </div>
 
         <!-- Dask Dashboard -->
-        <div class="dashboard-panel">
+        <div class="dashboard-panel full-height">
             <div class="panel-header">
                 <span class="status-indicator" id="dask-status"></span>
                 Dask Dashboard
             </div>
-            <div class="panel-content">
-                <iframe src="http://localhost:8787" id="dask-iframe"></iframe>
+            <div class="panel-content" id="dask-content">
+                <div style="color: #94a3b8; text-align: center; padding: 2rem;">
+                    Checking Dask availability...
+                </div>
             </div>
         </div>
     </div>
 
     <script>
+        // Fetch dashboard configuration once to get actual ports
+        let dashboardConfig = {
+            ray_dashboard_url: 'http://localhost:8265',
+            dask_dashboard_url: 'http://localhost:8787'
+        };
+
+        (async () => {
+            try {
+                const configRes = await fetch('/api/dashboard/config');
+                if (configRes.ok) {
+                    dashboardConfig = await configRes.json();
+                }
+            } catch (e) {
+                console.warn('Could not fetch dashboard config, using defaults');
+            }
+        })();
+
         // Update metrics every 2 seconds
         setInterval(async () => {
             try {
@@ -1204,55 +1298,109 @@ UNIFIED_DASHBOARD_HTML = """
                 document.getElementById('traces-json').textContent =
                     JSON.stringify(traces, null, 2);
 
-                // Network Nodes (Ollama)
+                // Network Nodes (Ollama) - Use table view for >5 nodes
                 const nodesRes = await fetch('/api/network/nodes');
                 const nodesData = await nodesRes.json();
                 const nodesList = document.getElementById('network-nodes');
 
                 if (nodesData.nodes && nodesData.nodes.length > 0) {
+                    const useTable = nodesData.nodes.length > 5;
                     let html = '';
-                    nodesData.nodes.forEach(node => {
-                        const isHealthy = node.status === 'healthy';
-                        const statusClass = isHealthy ? 'status-healthy' : 'status-offline';
-                        const cardClass = isHealthy ? 'node-card' : 'node-card offline';
 
-                        html += `<div class="${cardClass}">`;
-                        html += `  <div class="node-url">${node.url}</div>`;
-                        html += `  <div class="node-stats">`;
-                        html += `    <span class="status-badge ${statusClass}">${node.status}</span>`;
-                        html += `    <span class="stat-badge">⏱️ ${node.latency_ms}ms</span>`;
-                        html += `    <span class="stat-badge">📊 Load: ${node.load_percent}%</span>`;
-                        html += `    <span class="stat-badge">💾 ${node.memory_mb}MB</span>`;
-                        html += `  </div>`;
-                        html += `</div>`;
-                    });
+                    if (useTable) {
+                        // Compact table view for many nodes
+                        html = '<table class="compact-table"><thead><tr>';
+                        html += '<th>Host</th><th>Status</th><th>Latency</th><th>Load</th><th>Memory</th>';
+                        html += '</tr></thead><tbody>';
+
+                        nodesData.nodes.forEach(node => {
+                            const isHealthy = node.status === 'healthy';
+                            const statusClass = isHealthy ? 'status-healthy' : 'status-offline';
+                            const statusIcon = isHealthy ? '✅' : '❌';
+
+                            html += '<tr>';
+                            html += `<td style="color: #a78bfa; font-weight: 600;">${node.url}</td>`;
+                            html += `<td><span class="status-badge ${statusClass}">${statusIcon} ${node.status}</span></td>`;
+                            html += `<td>${node.latency_ms}ms</td>`;
+                            html += `<td>${node.load_percent}%</td>`;
+                            html += `<td>${node.memory_mb}MB</td>`;
+                            html += '</tr>';
+                        });
+
+                        html += '</tbody></table>';
+                    } else {
+                        // Card view for few nodes
+                        nodesData.nodes.forEach(node => {
+                            const isHealthy = node.status === 'healthy';
+                            const statusClass = isHealthy ? 'status-healthy' : 'status-offline';
+                            const cardClass = isHealthy ? 'node-card' : 'node-card offline';
+
+                            html += `<div class="${cardClass}">`;
+                            html += `  <div class="node-url">${node.url}</div>`;
+                            html += `  <div class="node-stats">`;
+                            html += `    <span class="status-badge ${statusClass}">${node.status}</span>`;
+                            html += `    <span class="stat-badge">⏱️ ${node.latency_ms}ms</span>`;
+                            html += `    <span class="stat-badge">📊 Load: ${node.load_percent}%</span>`;
+                            html += `    <span class="stat-badge">💾 ${node.memory_mb}MB</span>`;
+                            html += `  </div>`;
+                            html += `</div>`;
+                        });
+                    }
+
                     nodesList.innerHTML = html;
                 } else {
                     nodesList.innerHTML = '<div style="color: #94a3b8; text-align: center; padding: 2rem;">No Ollama nodes discovered</div>';
                 }
 
-                // RPC Backends (llama.cpp)
+                // RPC Backends (llama.cpp) - Use table view for >5 backends
                 const backendsRes = await fetch('/api/network/backends');
                 const backendsData = await backendsRes.json();
                 const backendsList = document.getElementById('rpc-backends');
 
                 if (backendsData.backends && backendsData.backends.length > 0) {
+                    const useTable = backendsData.backends.length > 5;
                     let html = '';
-                    backendsData.backends.forEach(backend => {
-                        const isHealthy = backend.status === 'healthy';
-                        const statusClass = isHealthy ? 'status-healthy' : 'status-offline';
-                        const cardClass = isHealthy ? 'backend-card' : 'backend-card offline';
 
-                        html += `<div class="${cardClass}">`;
-                        html += `  <div class="backend-url">${backend.url}</div>`;
-                        html += `  <div class="backend-stats">`;
-                        html += `    <span class="status-badge ${statusClass}">${backend.status}</span>`;
-                        html += `    <span class="stat-badge">⏱️ ${backend.latency_ms}ms</span>`;
-                        html += `    <span class="stat-badge">📨 Requests: ${backend.request_count || 0}</span>`;
-                        html += `    <span class="stat-badge">❌ Failures: ${backend.failure_count || 0}</span>`;
-                        html += `  </div>`;
-                        html += `</div>`;
-                    });
+                    if (useTable) {
+                        // Compact table view for many backends
+                        html = '<table class="compact-table"><thead><tr>';
+                        html += '<th>Backend</th><th>Status</th><th>Latency</th><th>Requests</th><th>Failures</th>';
+                        html += '</tr></thead><tbody>';
+
+                        backendsData.backends.forEach(backend => {
+                            const isHealthy = backend.status === 'healthy';
+                            const statusClass = isHealthy ? 'status-healthy' : 'status-offline';
+                            const statusIcon = isHealthy ? '✅' : '❌';
+
+                            html += '<tr>';
+                            html += `<td style="color: #a78bfa; font-weight: 600;">${backend.url}</td>`;
+                            html += `<td><span class="status-badge ${statusClass}">${statusIcon} ${backend.status}</span></td>`;
+                            html += `<td>${backend.latency_ms}ms</td>`;
+                            html += `<td>${backend.request_count || 0}</td>`;
+                            html += `<td style="color: ${backend.failure_count > 0 ? '#ef4444' : '#94a3b8'};">${backend.failure_count || 0}</td>`;
+                            html += '</tr>';
+                        });
+
+                        html += '</tbody></table>';
+                    } else {
+                        // Card view for few backends
+                        backendsData.backends.forEach(backend => {
+                            const isHealthy = backend.status === 'healthy';
+                            const statusClass = isHealthy ? 'status-healthy' : 'status-offline';
+                            const cardClass = isHealthy ? 'backend-card' : 'backend-card offline';
+
+                            html += `<div class="${cardClass}">`;
+                            html += `  <div class="backend-url">${backend.url}</div>`;
+                            html += `  <div class="backend-stats">`;
+                            html += `    <span class="status-badge ${statusClass}">${backend.status}</span>`;
+                            html += `    <span class="stat-badge">⏱️ ${backend.latency_ms}ms</span>`;
+                            html += `    <span class="stat-badge">📨 Requests: ${backend.request_count || 0}</span>`;
+                            html += `    <span class="stat-badge">❌ Failures: ${backend.failure_count || 0}</span>`;
+                            html += `  </div>`;
+                            html += `</div>`;
+                        });
+                    }
+
                     backendsList.innerHTML = html;
                 } else {
                     backendsList.innerHTML = '<div style="color: #94a3b8; text-align: center; padding: 2rem;">No RPC backends discovered</div>';
@@ -1292,22 +1440,42 @@ UNIFIED_DASHBOARD_HTML = """
                     appsList.innerHTML = '<div style="color: #94a3b8; text-align: center; padding: 2rem;">No applications registered yet</div>';
                 }
 
-                // Check Ray status
+                // Check Ray status and load iframe if available
                 try {
-                    const rayRes = await fetch('/api/ray/metrics');
-                    document.getElementById('ray-status').className =
-                        rayRes.ok ? 'status-indicator status-active' : 'status-indicator status-inactive';
+                    const rayRes = await fetch('/api/ray/metrics', {timeout: 1000});
+                    const rayContent = document.getElementById('ray-content');
+
+                    if (rayRes.ok) {
+                        document.getElementById('ray-status').className = 'status-indicator status-active';
+                        if (!rayContent.querySelector('iframe')) {
+                            rayContent.innerHTML = `<iframe src="${dashboardConfig.ray_dashboard_url}" style="width:100%;height:100%;border:none;"></iframe>`;
+                        }
+                    } else {
+                        document.getElementById('ray-status').className = 'status-indicator status-inactive';
+                        rayContent.innerHTML = '<div style="color: #f59e0b; text-align: center; padding: 2rem; background: #1e293b; border-radius: 0.5rem; margin: 1rem;"><div style="font-size: 1.5rem; margin-bottom: 0.5rem;">⚠️</div><div style="font-weight: 600; margin-bottom: 0.5rem;">Ray not initialized</div><small style="color: #94a3b8;">Use RayHybridRouter or RayAdvancedRouter to enable</small></div>';
+                    }
                 } catch {
                     document.getElementById('ray-status').className = 'status-indicator status-inactive';
+                    document.getElementById('ray-content').innerHTML = '<div style="color: #f59e0b; text-align: center; padding: 2rem; background: #1e293b; border-radius: 0.5rem; margin: 1rem;"><div style="font-size: 1.5rem; margin-bottom: 0.5rem;">⚠️</div><div style="font-weight: 600; margin-bottom: 0.5rem;">Ray not initialized</div><small style="color: #94a3b8;">Use RayHybridRouter or RayAdvancedRouter to enable</small></div>';
                 }
 
-                // Check Dask status
+                // Check Dask status and load iframe if available
                 try {
-                    const daskRes = await fetch('/api/dask/metrics');
-                    document.getElementById('dask-status').className =
-                        daskRes.ok ? 'status-indicator status-active' : 'status-indicator status-inactive';
+                    const daskRes = await fetch('/api/dask/metrics', {timeout: 1000});
+                    const daskContent = document.getElementById('dask-content');
+
+                    if (daskRes.ok) {
+                        document.getElementById('dask-status').className = 'status-indicator status-active';
+                        if (!daskContent.querySelector('iframe')) {
+                            daskContent.innerHTML = `<iframe src="${dashboardConfig.dask_dashboard_url}" style="width:100%;height:100%;border:none;"></iframe>`;
+                        }
+                    } else {
+                        document.getElementById('dask-status').className = 'status-indicator status-inactive';
+                        daskContent.innerHTML = '<div style="color: #f59e0b; text-align: center; padding: 2rem; background: #1e293b; border-radius: 0.5rem; margin: 1rem;"><div style="font-size: 1.5rem; margin-bottom: 0.5rem;">⚠️</div><div style="font-weight: 600; margin-bottom: 0.5rem;">Dask not initialized</div><small style="color: #94a3b8;">Enable with enable_dask=True in UnifiedDashboard</small></div>';
+                    }
                 } catch {
                     document.getElementById('dask-status').className = 'status-indicator status-inactive';
+                    document.getElementById('dask-content').innerHTML = '<div style="color: #f59e0b; text-align: center; padding: 2rem; background: #1e293b; border-radius: 0.5rem; margin: 1rem;"><div style="font-size: 1.5rem; margin-bottom: 0.5rem;">⚠️</div><div style="font-weight: 600; margin-bottom: 0.5rem;">Dask not initialized</div><small style="color: #94a3b8;">Enable with enable_dask=True in UnifiedDashboard</small></div>';
                 }
 
             } catch (error) {
